@@ -12,6 +12,7 @@ from ..aid.sixing import *
 from .globaling import *
 from ..aid.odicting import odict
 from ..aid import aiding
+from ..aid.timing import tuuid
 from . import excepting
 from . import registering
 from . import storing
@@ -19,7 +20,7 @@ from . import acting
 from . import tasking
 from . import framing
 
-from .consoling import getConsole
+from ..aid.consoling import getConsole
 console = getConsole()
 
 class Need(acting.Actor):
@@ -27,6 +28,23 @@ class Need(acting.Actor):
     """
     Registry = odict()
 
+    def __init__(self, **kwa):
+        """
+        Initialize Instance
+
+        Inherited Attributes:
+            .name = name string for Actor variant in class Registry
+            .store = reference to shared data Store
+            ._act = reference to containing Act
+
+        Attributes:
+            ._tracts = list of references to transition acts for this need
+                transit sub-context of precur context during segue
+
+        """
+        super(Need, self).__init__(**kwa)
+        self._tracts = []
+
     def _expose(self):
         """
         """
@@ -60,6 +78,15 @@ class Need(acting.Actor):
 
         return result
 
+    def addTract(self, act):
+        """
+        Add act to ._tracts list
+        """
+        self._tracts.append(act)
+        act.frame = self._act.frame.name  # re-resolve later
+        act.context = ActionSubContextNames[TRANSIT]
+
+
 class NeedAlways(Need):
     """NeedAlways Need  Special Need"""
     def action(self, **kw):
@@ -69,12 +96,46 @@ class NeedAlways(Need):
         return result
 
 class NeedDone(Need):
-    """NeedDone Need Special Need"""
-    def _resolve(self, tasker, framer, frame, **kwa):
+    """
+    NeedDone Need Special Need
+    """
+    def _resolve(self, tasker, **kwa):
         """Resolves value (tasker) link that is passed in as tasker parm
            resolved link is passed back to container ._act to update in act's parms
         """
         parms = super(NeedDone, self)._resolve( **kwa)
+
+        parms['tasker'] = tasker = tasking.resolveTasker(tasker,
+                                                        who=self._act.frame.name,
+                                                        desc='need done',
+                                                        contexts=[],
+                                                        human=self._act.human,
+                                                        count=self._act.count)
+
+        return parms #return items are updated in original ._act parms
+
+    def action(self, tasker, **kw):
+        """
+        Check if  tasker done
+        parameters:
+            tasker
+        """
+        result = tasker.done
+        console.profuse("Need Tasker {0} done = {1}\n".format(tasker.name, result))
+        return result
+
+class NeedDoneAux(Need):
+    """
+    NeedDoneAux Need Special Need
+    """
+    def _resolve(self, tasker, framer, frame, **kwa):
+        """
+        Resolves aux framer tag link that is passed in as tasker parm relative to
+        containing framer and frame
+
+        Resolved link is passed back to container ._act to update in act's parms
+        """
+        parms = super(NeedDoneAux, self)._resolve( **kwa)
         if framer:
             if framer == 'me':
                 framer = self._act.frame.framer
@@ -95,35 +156,42 @@ class NeedDone(Need):
                                                                   human=self._act.human,
                                                                   count=self._act.count)
 
-
         if tasker not in ['any', 'all']:
-            parms['tasker'] = tasker = tasking.resolveTasker(tasker,
-                                                             who=self._act.frame.name,
-                                                             desc='need done',
-                                                             contexts=[],
-                                                             human=self._act.human,
-                                                             count=self._act.count)
-
+            parms['tasker'] = tasker = framing.resolveAuxOfFramer(tasker,
+                                                            framer,
+                                                            who=self._act.frame.name,
+                                                            desc='need done aux',
+                                                            contexts=[],
+                                                            human=self._act.human,
+                                                            count=self._act.count)
 
         return parms #return items are updated in original ._act parms
 
     def action(self, tasker, framer, frame, **kw):
         """
-        Check if  tasker done
+        Check if  aux done
         parameters:
-            tasker
+            aux
+            framer
+            frame
         """
         if frame:
             if tasker == 'any':
                 result = any([aux.done for aux in frame.auxes ])
             elif tasker == 'all':
                 result = frame.auxes and all([aux.done for aux in frame.auxes])
+            elif tasker in frame.auxes:
+                result = tasker.done
             else:
                 result = False
-
+            name = tasker if tasker in ('any', 'all') else tasker.tag
+            console.profuse("Need Aux {0} done = {1} in {2}<{3}\n".format(name,
+                                                                          result,
+                                                                          framer.name,
+                                                                          frame.name))
         else:
             result = tasker.done
-            console.profuse("Need Framer {0} done = {1}\n".format(tasker.name, result))
+            console.profuse("Need Aux {0} done = {1}\n".format(tasker.name, result))
 
         return result
 
@@ -153,6 +221,7 @@ class NeedStatus(Need):
           tasker
           status
         """
+        # maybe should add check for auxiliary since status never changes for auxiliary
 
         result = (tasker.status == status)
         console.profuse("Need Tasker {0} status is {1} = {2}\n".format(
@@ -310,120 +379,178 @@ class NeedMarker(Need):
     Special Need
 
     """
-    def _resolve(self, share, frame, marker, **kwa):
+    def _resolve(self, share, frame, kind, marker, **kwa):
         """
         Resolves frame name link and then
            inserts marker as first enact in the resolved frame
 
-        parms:
-            share       share ref holding mark
-            name        name of frame where marker is watching
-            frame       only used in resolvelinks
-            marker      only used in resolvelinks
+        Incoming Parameters:
+            share is path or ref of share holding mark
+            frame is name or ref of frame to add option enact marker
+                If frame empty then only put marker in transit sub context of
+                need act's frame
+            kind is name of marker actor class for marker act
+            marker is unique identifier of marker if not empty
+                This allows multiple if updated/changed to use same marker
+
+        Outgoing Parameters:
+            share is resolved ref of share holding mark
+            marker is unique key for Mark instance in .marks odict in share
+
+        Marker Act Parameters:
+            share is resolved ref of share holding mark
+            marker is unique key for Mark instance in .marks odict in share
 
         """
         parms = super(NeedMarker, self)._resolve( **kwa)
 
         framer = self._act.frame.framer
+        enacted = True if frame else False
 
-        if frame == 'me':
+        if not frame or frame == 'me':
             frame = self._act.frame
-
-        parms['frame'] = frame = framing.resolveFrameOfFramer(frame,
-                                                              framer,
-                                                              who=self.name,
-                                                              desc='need marker',
-                                                              human=self._act.human,
-                                                              count=self._act.count)
+        frame = framing.resolveFrameOfFramer(frame,
+                                                framer,
+                                                who=self.name,
+                                                desc='need marker',
+                                                human=self._act.human,
+                                                count=self._act.count)
 
         parms['share'] = share = self._resolvePath(ipath=share,
                                                   warn=True) # now a share
+        parts = [framer.name]
+        if marker:
+            parts.append(marker)  # framername.marker
+        else:
+            parts.append(frame.name)  # default is framername.framename
 
-        if not share.marks.get(frame.name):
-            share.marks[frame.name] = storing.Mark()
+        marker = "<".join(parts)
+        parms['marker'] = marker
 
-        found = False
-        for enact in frame.enacts:
-            if (isinstance(enact.actor, acting.Actor) and
-                    enact.actor.name is marker and
-                    enact.parms['share'] is share and
-                    enact.parms['name'] == frame.name):
-                found = True
-                break
+        if not share.marks.get(marker):
+            share.marks[marker] = storing.Mark()
 
-        if not found:
-            if marker not in acting.Actor.Registry:
-                msg = "ResolveError: Bad need marker link"
-                raise excepting.ResolveError(msg, marker, self.name,
-                                self._act.human, self._act.count)
+        if kind not in acting.Actor.Registry:
+            msg = "ResolveError: Bad need marker link"
+            raise excepting.ResolveError(msg, kind, self.name,
+                            self._act.human, self._act.count)
 
-            markerParms = dict(share=share, frame=frame.name)
-            parms['marker'] = marker = acting.Act(  actor=marker,
-                                                    registrar=acting.Actor,
-                                                    parms=markerParms,
-                                                    human=self._act.human,
-                                                    count=self._act.count)
+        markerParms = dict(share=share, marker=marker)
+        markerAct = acting.Act(actor=kind,
+                                registrar=acting.Actor,
+                                parms=markerParms,
+                                human=self._act.human,
+                                count=self._act.count)
 
-            frame.insertEnact(marker)
-            console.profuse("     Added {0} {1} with {2} in {3}\n".format(
-                'enact',
-                marker,
-                marker.parms['share'].name,
-                marker.parms['frame']))
-            marker.resolve()
+        self.addTract(markerAct)  # sets act.context to 'transit'
+        console.profuse("     Added {0} {1} with {2} at {3} in {4} of "
+                        "framer {5}\n".format(
+                                'tract',
+                                markerAct,
+                                markerAct.parms['share'].name,
+                                markerAct.parms['marker'],
+                                self._act.frame.name,
+                                framer.name))
+        markerAct.resolve()
+
+        if enacted:  # only add enact marker if original provided frame not empty
+            found = False
+            for enact in frame.enacts:  # avoid adding redundant marker
+                if (isinstance(enact.actor, acting.Actor) and
+                        enact.actor.name == kind and
+                        enact.parms['share'].name == share.name and
+                        enact.parms['marker'] == marker):
+                    found = True
+                    break
+
+            if not found:
+                markerParms = dict(share=share, marker=marker)
+                markerAct = acting.Act(actor=kind,
+                                                         registrar=acting.Actor,
+                                                         parms=markerParms,
+                                                         human=self._act.human,
+                                                         count=self._act.count)
+
+                frame.insertEnact(markerAct)
+                console.profuse("     Added {0} {1} with {2} at {3} in {4} of "
+                                "framer {5}\n".format(
+                                        'enact',
+                                        markerAct,
+                                        markerAct.parms['share'].name,
+                                        markerAct.parms['marker'],
+                                        frame.name,
+                                        framer.name))
+                markerAct.resolve()  # resolves .actor given by actor kind name into actor class
 
         return parms #return items are updated in original ._act parms
 
 class NeedUpdate(NeedMarker):
     """ NeedUpdate Need Special Need """
-    def action(self, share, frame, **kw):
+    def action(self, share, marker, **kw):
         """
-        Check if share updated while in frame/mark denoted by name key if any
-            Default is False
+        Check if share updated since mark in share was updated by marker
+        Default is False
 
-        parameters:
-            share = resolved share that is marked
-            frame = resolved frame where marker is placed
-            marker = marker kind name
+        Parameters:
+            share is resolved share that is marked with .mark[marker]
+            marker is marker key
         """
         result = False
-        mark = share.marks.get(frame.name) #get mark from mark frame name key
-        if mark and mark.stamp is not None and share.stamp is not None:
-            result = share.stamp >= mark.stamp #equals so catch updates on enter
+        mark = share.marks.get(marker)
+        if mark and share.stamp is not None:  # only if share updated from None
+            # share stamps only non-None if updated after starts running not init
+            # == catches updates to share on same enter or precur as Marker reset
+            # mark.used only lets == work the first time
+            # mark.stamp is None and share.stamp not None always True so if mark
+            # not yet set will always work first time
+            # store stamp is None until first run by skedder to share.stamp is
+            # not updated until some action updates it. (not by init)
+            result = ((mark.stamp is None) or
+                      (share.stamp > mark.stamp) or
+                      (share.stamp == mark.stamp and mark.used != mark.stamp))
 
-        console.profuse("Need Share {0} update in Frame {1} = {2}\n".format(
-            share.name, frame.name, result))
+        console.profuse("Marker update {0} for {1} of Share {2} {3} "
+                        " {4} mark {5} used {6} at {7}\n".format(result,
+                                                     marker,
+                                                     share.name,
+                                                     share.stamp,
+                                                     '>=',
+                                                     mark.stamp,
+                                                     mark.used,
+                                                     self.store.stamp))
 
         return result
 
 class NeedChange(NeedMarker):
     """NeedChange Need Special Need"""
-    def action(self, share, frame, **kw):
+    def action(self, share, marker, **kw):
         """
-        Check if share data changed while in frame/mark denoted by name key if any
-            Default is False
-        parameters:
-            share
-            name
-            frame       only used in resolvelinks
-            marker      only used in resolvelinks
+        Check if share data changed while denoted by marker key if any
+        Default is False
+
+        Parameters:
+            share is resolved share that is marked with .mark[marker]
+            marker is marker key
         """
         result = False
-        mark = share.marks.get(frame.name) #get mark from mark frame name key
-        if mark and mark.data is not None:
-            for field, value in share.items():
-                try:
-                    if getattr(mark.data, field) != value:
-                        result = True
+        mark = share.marks.get(marker) #get mark from mark frame name key
+        if mark:
+            if mark.data is None:
+                result = True  # always true first time if mark not yet set
+            else:
+                for field, value in share.items():
+                    try:
+                        if getattr(mark.data, field) != value:
+                            result = True
 
-                except AttributeError as ex: # new attribute so changed
-                    result = True
+                    except AttributeError as ex: # new attribute so changed
+                        result = True  # should not remove fields so only test add case
 
-                if result: #stop checking on first change
-                    break
+                    if result: #stop checking on first change
+                        break
 
 
-        console.profuse("Need Share {0} change in Frame {1} = {2}\n".format(
-            share.name, frame.name, result))
+        console.profuse("Marker change {0} for {1} of data {2} of share {3} at {4}\n".format(
+            result, marker, mark.data if mark else None, share.name, self.store.stamp))
 
         return result
